/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <msp430.h>
#include "driverlib.h"
#include "adc.h"

volatile int adc_results[4] = {0,0,0,0};
volatile int unsigned adc_vcc;
volatile int adc_results_flag=0;
volatile int adc_vcc_flag=0;

/**
 * Initialize ADC using Timer D0 to trigger a repeated sequence of channels (A3 .. A0)
 * The DMA copies the ADC data to memory
 */
void adc_init(void){
	ADC10_A_disableConversions(ADC10_A_BASE, ADC10_A_COMPLETECONVERSION);
	ADC10_A_init(ADC10_A_BASE, ADC10_A_SAMPLEHOLDSOURCE_3,	ADC10_A_CLOCKSOURCE_ADC10OSC, ADC10_A_CLOCKDIVIDER_1);
	ADC10_A_enable(ADC10_A_BASE);
	ADC10_A_configureMemory(ADC10_A_BASE, ADC10_A_INPUT_A3,	ADC10_A_VREFPOS_EXT, ADC10_A_VREFNEG_EXT);		// external reference on TIDA-01081

	DMA_initParam param = { 0 };
	param.channelSelect = DMA_CHANNEL_0;
	param.transferModeSelect = DMA_TRANSFER_REPEATED_SINGLE;
	param.transferSize = 4;
	param.triggerSourceSelect = DMA_TRIGGERSOURCE_24;
	param.transferUnitSelect = DMA_SIZE_SRCWORD_DSTWORD;
	param.triggerTypeSelect = DMA_TRIGGER_RISINGEDGE;
	DMA_init(&param);
	DMA_setSrcAddress(DMA_CHANNEL_0, ADC10_A_getMemoryAddressForDMA(ADC10_A_BASE), DMA_DIRECTION_UNCHANGED);
	DMA_setDstAddress(DMA_CHANNEL_0, (uint32_t) &adc_results[0], DMA_DIRECTION_INCREMENT);
	DMA_clearInterrupt(DMA_CHANNEL_0);
	DMA_enableTransfers(DMA_CHANNEL_0);

	ADC10_A_startConversion(ADC10_A_BASE, ADC10_A_REPEATED_SEQOFCHANNELS);

	TD0CTL0 = MC_1 | TDCLR | TDSSEL__ACLK | TDHD1|TDHD0; // count up, div by 8, aclk
	TD0CCR0 = 128;
	TD0CCR1 = 128;
	TD0CCTL1 = OUTMOD_4; // toggle compare output
}

/**
 * Get the VCC voltage by polling the ADC
 * @return ADC raw value from VCC channel
 */
int adc_poll_vcc(void){
	ADC10_A_init(ADC10_A_BASE, ADC10_A_SAMPLEHOLDSOURCE_SC,	ADC10_A_CLOCKSOURCE_ADC10OSC, ADC10_A_CLOCKDIVIDER_1);
	ADC10_A_enable(ADC10_A_BASE);
	ADC10_A_setupSamplingTimer(ADC10_A_BASE, ADC10_A_CYCLEHOLD_16_CYCLES, ADC10_A_MULTIPLESAMPLESDISABLE);
	ADC10_A_configureMemory(ADC10_A_BASE, ADC10_A_INPUT_BATTERYMONITOR,	ADC10_A_VREFPOS_INT, ADC10_A_VREFNEG_AVSS);	// internal reference

	// using the internal reference, as the 2.5V external might not be stable at this point
	while(REF_BUSY == Ref_isRefGenBusy(REF_BASE));
	Ref_setReferenceVoltage(REF_BASE, REF_VREF2_0V);
	Ref_enableReferenceVoltage(REF_BASE);
	__delay_cycles(75);

	ADC10_A_startConversion(ADC10_A_BASE, ADC10_A_SINGLECHANNEL);
	while(ADC10_A_isBusy(ADC10_A_BASE));
	return ADC10_A_getResults(ADC10_A_BASE);
}
