/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <msp430.h>
#include "driverlib.h"
#include "dac.h"

unsigned int led_current=200;
int led_current_flag=0;

#define SCLKPORT	P3OUT
#define SCLKPIN		(1<<3)
#define SDOUTPORT	P3OUT
#define SDOUTPIN	(1<<4)
#define DAC73CSPORT	PJOUT
#define DAC73CSPIN	(1<<6)
#define DAC082CSPORT	PJOUT
#define DAC082CSPIN		(1<<5)
#define HYST			5

/*static const char vin_efuse[] = {9, 20, 30, 37, 43, 48, 52, 56, 59, 61, 64, 66, 68, 70,
		71, 73, 74, 75, 76, 77, 78, 79, 80, 81, 81, 82, 83, 83, 84, 84}; //lookup table efuse configuration depending on vin, from 7 to 36 V in 1 V step*/

static const char vin_efuse[] = {0, 0, 3, 15, 24, 31, 38, 43, 48, 52, 55, 58, 61, 63,
		66, 68, 70, 71, 73, 74, 76, 77, 78, 79, 80, 81, 82, 83, 83, 84}; //lookup table efuse configuration depending on vin, from 7 to 36 V in 1 V step


/*static const char vin_iboost[] = {255, 230, 205, 184, 167, 153, 142, 132, 123, 115, 108,
		102, 97, 92, 88, 84, 80, 77, 74, 71, 68, 66, 64, 61, 59, 58, 56, 54, 53, 51}; //lookup table iboost configuration depending on vin, from 7 to 36 V in 1 V step*/

static const char vin_iboost[] = {255, 255, 255, 236, 212, 193, 177, 164, 152, 142, 133,
		125, 118, 112, 106, 101, 96, 92, 88, 84, 81, 78, 75, 72, 69, 67, 65, 63, 61, 59}; //lookup table iboost configuration depending on vin, from 7 to 36 V in 1 V step

/**
 * Shift out 16 bits of data using softspi
 * @param data data to be shifted
 */

static void spi_shift(int data){
	int i;

	for(i=15;i>=0;i--){
		SCLKPORT &= ~SCLKPIN;
		if(data & (1<<i))
			SDOUTPORT |= SDOUTPIN;
		else
			SDOUTPORT &= ~SDOUTPIN;
		SCLKPORT |= SCLKPIN;
	}
	SCLKPORT &= ~SCLKPIN;
}

/**
 * Set DAC7311 to a given value
 * Used for setting LED current
 * @param data value to be set
 */
inline void dac73_set(int data){
	int dac = data + 120; // current set by dac is peak current

	DAC73CSPORT &= ~DAC73CSPIN;
	spi_shift((dac <<2) & 0x3fff);
	DAC73CSPORT |= DAC73CSPIN;
}

/**
 * Set DAC082S085 channels to a specific value
 * Channel 0: EFuse
 * Channel 1: Boost
 * @param channel channel to be set
 * @param data value to set
 */
void dac082_set(int channel, int data){
	DAC082CSPORT &= ~DAC082CSPIN;
	spi_shift(channel<<14 | data<<4 | (1<<12)); // set channel, data and update outputs
	DAC082CSPORT |= DAC082CSPIN;
}

/**
 * Set efuse current limit depending on input voltage
 * @param vin meassured input voltage in volts*10
 */
void efuse_setlimit(int vin){
	static int vin_prev=0;
	if(abs(vin-vin_prev) < HYST) return;
	vin_prev = (vin/5)*5; // round to .5V

	int voltage = vin/10;
	if(vin%10 > 5) voltage++;

	if(voltage < 7) voltage = 7;
	if(voltage > 36) voltage = 36;
	dac082_set(DACEFUSE, vin_efuse[voltage-7]);
}

/**
 * Set boost current depending on input voltage
 * @param vin meassured input voltage in volts
 */
void iboost_setlimit(int vin){
	static int vin_prev=0;
	if(abs(vin-vin_prev) < HYST) return;
	vin_prev = (vin/5)*5; // round to .5V

	int voltage = vin/10;
	if(vin%10 > 5) voltage++;
	if(voltage < 7) voltage = 7;
	if(voltage > 36) voltage = 36;
	dac082_set(DACBOOST, vin_iboost[voltage-7]);
}
