/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#ifndef DRV_VT100_H_
#define DRV_VT100_H_

enum field_enum{ontime_field=2, led_temp_field=3, framerate_field=4,
	diodes_temp_field=5, trigger_field=6, buck_temp_field=7, nflashes_field=8,
	inductor_temp_field=9, led_current_field=10,
	vin_field=13, efuse_dac_field=14, iin_field=15, boost_dac_field=16,
	vboost_field=17, buck_dac_field=18, vled_field=19, vcc_field=20,
	buck_ovp_field=22, state_field=23, boost_ovp_field=24, efuse_field=25,
	eeprom_field=26, loadsw_field=27, eeprom_data_field=28, flash_bracketing_field=29,
	monitoring_field=30, overtemp_field=31, fail_field=32, current_bracketing_field=33};

/**
 * @brief Structure for building the VT100 menu over UART
 */
struct menu_struct{
	const char *text; 	/** Title of entry */
	const int startx; 	/** x position */
	const int starty; 	/** y position */
	char *value; 		/** pointer to char array storing displayed value */
};

extern const struct menu_struct menu[];
extern int menu_init_state;
extern int monitoring_state;

void vt100_init(void);
void vt100_setcur(int x, int y);
void vt100_printf(int x, int y, const char * format, ...);
void menu_init(void);
void menu_write_field(enum field_enum field);
void menu_update_field(enum field_enum field);
void menu_input(char c);


#endif /* DRV_VT100_H_ */
