/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <msp430.h> 
#include "driverlib.h"

#include "main.h"
#include "sanity.h"
#include "drv/timer.h"
#include "drv/uart.h"
#include "drv/pinmux.h"
#include "drv/adc.h"
#include "drv/vt100.h"
#include "drv/i2c.h"
#include "drv/dac.h"

#include <stdio.h>
#include <stdlib.h>

int main(void) {
	int data[4];
	char operation_state_old=1, trigger_src_old=1, monitoring_state_old, led_temp_old=0,
			diode_temp_old=0, buck_conv_temp_old=0, buck_ind_temp_old=0,
			state_old=0x55, state, boost_fail_cnt=0;
	int i, vin_old=0, iin_old=0, vboost_old=0, vled_old=0, vin_prev=0, error_old=0, timeout=1000;

    WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer

	pinmux_init();
	iboost_setlimit(360);
	//while(rising){
	//	if(adc_results[ADC_VIN_CHN] != vin_prev) adc_raw[adc_pos++] = adc_results[ADC_VIN_CHN];
	//	if(adc_pos == 128){
	//		rising = 0;
	//	}
	/*while(adc_results[ADC_VIN_CHN]-vin_prev > 10 || adc_results[ADC_VIN_CHN] == vin_prev){
		vin_prev = adc_results[ADC_VIN_CHN];
	}*/
	while(adc_poll_vcc() < ((30*1024)/40));
	adc_init();
	while(ADC_VIN_RES == 0);
	efuse_setlimit(ADC_VIN_RES);
	load_sw(1);

	//Set VCore = 3 for 25MHz clock
    PMM_setVCore(PMM_CORE_LEVEL_3);
	//Set DCO FLL reference = REFO
	UCS_initClockSignal(UCS_FLLREF, UCS_REFOCLK_SELECT, UCS_CLOCK_DIVIDER_1);
	//Set ACLK = REFO
	UCS_initClockSignal(UCS_ACLK, UCS_REFOCLK_SELECT, UCS_CLOCK_DIVIDER_1);
	//Set Ratio and Desired MCLK Frequency  and initialize DCO
	UCS_initFLLSettle(25000, 762);

	__delay_cycles(65535);
	__delay_cycles(65535);
	__delay_cycles(65535);
	__delay_cycles(65535);
	P2OUT &= ~(1<<0); // reset OVP
	P2OUT |= (1<<0); // reset OVP
	// clear flags after turning on load switch
	P2IFG &= ~(1<<4);
	P2IFG &= ~(1<<7);

	i2c_init();
	timer_init();
	uart_init();
	menu_init_state = 2;
	menu_init();
	__enable_interrupt();
	__bis_SR_register(SCG0); //disable FLL loop to reduce jitter

	ontime_flag=1;
	framerate_flag=1;
	eeprom_select_flag=1;
	eeprom_change_flag=1;
	led_current_flag=1;
	flash_count_flag=1;
	flash_bracketing_flag=1;
	current_bracketing_flag=1;

	while(ADC_VBOOST_RES < 470 && timeout){
		__delay_cycles(65535);
		timeout--;
	}

	while(1){
#ifdef DEBUG
		PJOUT |= (1<<2);
#endif
		if(input_buffer_pos){
			for(i=0;i<input_buffer_pos;i++){
				menu_input(input_buffer[i]);
			}
			input_buffer_pos = 0;
		}
		if(monitoring_state){
			if(vin_old != adc_results[ADC_VIN_CHN]){
				sprintf(menu[vin_field].value, "%3d.%d V", ADC_VIN_RES/10, ADC_VIN_RES%10); // 500us
				menu_update_field(vin_field); // rather slow ~1ms
				vin_old = adc_results[ADC_VIN_CHN];
			}

			if(abs(vin_prev-adc_results[ADC_VIN_CHN]) < 10){
				efuse_setlimit(ADC_VIN_RES); // control efuse depending on vcc
				iboost_setlimit(ADC_VIN_RES); // control boost
			}
			vin_prev = adc_results[ADC_VIN_CHN];

			if(iin_old != adc_results[ADC_IIN_CHN]){
				sprintf(menu[iin_field].value, "%3d.%03d A ", ADC_IIN_RES/1000, ADC_IIN_RES%10000);
				menu_update_field(iin_field);
				iin_old = adc_results[ADC_IIN_CHN];
			}

			if(vboost_old != adc_results[ADC_VBOOST_CHN] || boost_fail_cnt > 0){
				sprintf(menu[vboost_field].value, "%3d.%d V", ADC_VBOOST_RES/10, ADC_VBOOST_RES%10);
				menu_update_field(vboost_field);
				if(ADC_VBOOST_RES < 360){
					if(boost_fail_cnt++ == 2){
						operation_state = 0;
						error = fail_boost;
					}
				}else
					boost_fail_cnt = 0;
				vboost_old = adc_results[ADC_VBOOST_CHN];
			}

			if(vled_old != adc_results[ADC_VLED_CHN]){
				sprintf(menu[vled_field].value, "%3d.%d V", ADC_VBUCK_RES/10, ADC_VBUCK_RES%10);
				menu_update_field(vled_field);
				vled_old = adc_results[ADC_VLED_CHN];
			}

			if(error_old != error){
				sprintf(menu[fail_field].value, "%s", error_text[error]);
				menu_update_field(fail_field);
				error_old = error;
			}

			state = P2IN & ((1<<1)|(1<<4));
			state |= (PJIN & (1<<4)) ? 1 : 0;
			state |= (P3IN & (1<<2)) ? (1<<2) : 0;
			if(state != state_old){
				sprintf(menu[buck_ovp_field].value, (P2IN & (1<<1)) ? "fault": "ok   ");
				menu_update_field(buck_ovp_field);

				sprintf(menu[boost_ovp_field].value, (PJIN & (1<<4)) ? "fault": "ok   ");
				menu_update_field(boost_ovp_field);

				sprintf(menu[efuse_field].value, (P2IN & (1<<4)) ? "ok   " : "fault");
				menu_update_field(efuse_field);

				sprintf(menu[overtemp_field].value, (P3IN & (1<<2)) ? "ok   " : "fault");
				menu_update_field(overtemp_field);

				state_old = state;
			}

			if(!(P3IN & (1<<2))) reset_temp_error(); // reset temperature error

			if(temp_timer > 2500){
				poll_temp();
				temp_timer = 0;
			}

			if(temp_flag){
				temp_flag = 0;
				if(led_temp != led_temp_old){
					sprintf(menu[led_temp_field].value, "%3d C", led_temp);
					menu_update_field(led_temp_field);
					led_temp_old = led_temp;
				}
				if(diode_temp != diode_temp_old){
					sprintf(menu[diodes_temp_field].value, "%3d C", diode_temp);
					menu_update_field(diodes_temp_field);
					diode_temp_old = diode_temp;
				}
				if(buck_conv_temp != buck_conv_temp_old){
					sprintf(menu[buck_temp_field].value, "%3d C", buck_conv_temp);
					menu_update_field(buck_temp_field);
					buck_conv_temp_old = buck_conv_temp;
				}
				if(buck_ind_temp != buck_ind_temp_old){
					sprintf(menu[inductor_temp_field].value, "%3d C", buck_ind_temp);
					menu_update_field(inductor_temp_field);
					buck_ind_temp_old = buck_ind_temp;
				}
			}
		}

		if(!(P3IN & (1<<2))) { // alert pin from TMP116
			operation_state = 0;
			monitoring_state = 1;
			error = fail_temp;
		}

		if(ontime_flag){
			ontime_flag = 0;
			char unit;
			switch(timebaserange){
			case 0: unit = 'n';	break;
			case 1: unit = 'u'; break;
			case 2: unit = 'm'; break;
			case 3: unit = ' '; break;
			}
			check_ontime();
			timer_d_configure(ontime, led_current, timebaserange);
			sprintf(menu[ontime_field].value, "%5d %c", ontime, unit);
			menu_write_field(ontime_field);
			led_current_flag = 1; // check and print current, might has change
			framerate_flag = 1;
		}
		if(framerate_flag){
			framerate_flag = 0;
			check_framerate();
			if(framerate > 1000)
				sprintf(menu[framerate_field].value, "%5d.%d Hz", 10000/framerate, (int)(100000UL/framerate)%10);
			else if(framerate > 0 && framerate <= 1000)
				sprintf(menu[framerate_field].value, "%5d Hz  ", 10000/framerate);
			else
				sprintf(menu[framerate_field].value, "%5d Hz  ", 0);
			menu_write_field(framerate_field);
			led_current_flag = 1; // check and print current, might has change
		}

		if(eeprom_select_flag){
			eeprom_select_flag = 0;
			if(eeprom_select > 3) eeprom_select = 3;
			if(eeprom_select < 0) eeprom_select = 0;
			read_eeprom(data);
			sprintf(menu[eeprom_data_field].value, "%04x %04x %04x %04x", data[0], data[1], data[2], data[3]);
			menu_write_field(eeprom_data_field);
		}
		if(eeprom_change_flag){
			eeprom_change_flag = 0;
			sscanf(menu[eeprom_data_field].value, "%04x %04x %04x %04x", &data[0], &data[1], &data[2], &data[3]);
			write_eeprom_all(data);
			eeprom_select_flag = 1; // trigger read
		}
		if(led_current_flag){
			led_current_flag = 0;
			check_current();
			timer_d_configure(ontime, led_current, timebaserange);
			sprintf(menu[led_current_field].value, "%5d mA", led_current);
			menu_write_field(led_current_field);
			dac73_set(led_current);
			if(framerate == 0 && trigger_src == 0){
				dc_mode(operation_state);
			}
		}
		if(operation_state != operation_state_old){
			operation_state_old = operation_state;
			if(operation_state) { // turn on
				timer_d_configure(ontime, led_current, timebaserange);
				error = fail_none;
				flash_counter = 0; // reset pulse counter at start
				flash_bracketing_counter = 0; // reset bracketing counter
				if(trigger_src) timer_init_ext(); // switch to timeout based timing in ext mode
				running = 1;
			}else{
				running = 0;
				if(framerate == 0) dc_mode(0);
				timer_init(); //normal timing when off
			}
			sprintf(menu[state_field].value, operation_state?"On ":"Off");
			menu_write_field(state_field);
			if(framerate == 0 && trigger_src == 0){
				dc_mode(operation_state);
			}
		}
		if(trigger_src != trigger_src_old){
			sprintf(menu[trigger_field].value, trigger_src?"Ext":"Int");
			menu_write_field(trigger_field);
			trigger_src_old = trigger_src;
		}
		if(flash_count_flag){
			sprintf(menu[nflashes_field].value, "%4d", flash_count);
			menu_write_field(nflashes_field);
			flash_count_flag = 0;
		}
		if(flash_bracketing_flag){
			sprintf(menu[flash_bracketing_field].value, "%s", flash_bracketing_text[flash_bracketing]);
			menu_write_field(flash_bracketing_field);
			flash_bracketing_flag = 0;
		}
		if(monitoring_state != monitoring_state_old){
			sprintf(menu[monitoring_field].value, monitoring_state?"On ":"Off");
			menu_write_field(monitoring_field);
			monitoring_state_old = monitoring_state;
		}
		if(current_bracketing_flag){
			sprintf(menu[current_bracketing_field].value, "%5d mA", current_bracketing);
			menu_write_field(current_bracketing_field);
			current_bracketing_flag = 0;
		}

#ifdef DEBUG
		PJOUT &= ~(1<<2);
#endif
		__bis_SR_register(LPM0_bits + GIE);
	}
}







